@@ -131,7 +131,7 @@ vector<pair<vector<DNSRecord>, vector<DNSRecord> > > getIXFRDeltas(const ComboAd
   DNSPacketWriter pw(packet, zone, QType::IXFR);
   pw.getHeader()->qr=0;
   pw.getHeader()->rd=0;
-  pw.getHeader()->id=dns_random(0xffff);
+  pw.getHeader()->id=dns_random_uint16();
   pw.startRecord(zone, QType::SOA, 0, QClass::IN, DNSResourceRecord::AUTHORITY);
   oursr.d_content->toPacket(pw);
 
@@ -146,7 +146,7 @@ vector<pair<vector<DNSRecord>, vector<DNSRecord> > > getIXFRDeltas(const ComboAd
     } catch(PDNSException& pe) {
       throw std::runtime_error("TSIG algorithm '"+tt.algo.toLogString()+"' is unknown.");
     }
-    trc.d_time = time((time_t*)NULL);
+    trc.d_time = time((time_t*)nullptr);
     trc.d_fudge = 300;
     trc.d_origID=ntohs(pw.getHeader()->id);
     trc.d_eRcode=0;
