@@ -5,7 +5,6 @@
 #include "config.h"
 #endif
 #include <boost/test/unit_test.hpp>
-#include <boost/test/floating_point_comparison.hpp>
 #include "mtasker.hh"
 
 BOOST_AUTO_TEST_SUITE(mtasker_cc)
@@ -15,25 +14,27 @@ static int g_result;
 static void doSomething(void* p)
 {
   MTasker<>* mt = reinterpret_cast<MTasker<>*>(p);
-  int i=12, o;
+  int i = 12, o;
   if (mt->waitEvent(i, &o) == 1)
     g_result = o;
 }
 
-BOOST_AUTO_TEST_CASE(test_Simple) {
+BOOST_AUTO_TEST_CASE(test_Simple)
+{
   MTasker<> mt;
   mt.makeThread(doSomething, &mt);
   struct timeval now;
   gettimeofday(&now, 0);
-  bool first=true;
-  int o=24;
-  for(;;) {
-    while(mt.schedule(&now));
-    if(first) {
+  bool first = true;
+  int o = 24;
+  for (;;) {
+    while (mt.schedule(&now))
+      ;
+    if (first) {
       mt.sendEvent(12, &o);
-      first=false;
+      first = false;
     }
-    if(mt.noProcesses())
+    if (mt.noProcesses())
       break;
   }
   BOOST_CHECK_EQUAL(g_result, o);
@@ -44,16 +45,17 @@ static void willThrow(void* p)
   throw std::runtime_error("Help!");
 }
 
+BOOST_AUTO_TEST_CASE(test_MtaskerException)
+{
+  BOOST_CHECK_THROW({
+    MTasker<> mt;
+    mt.makeThread(willThrow, 0);
+    struct timeval now;
 
-BOOST_AUTO_TEST_CASE(test_MtaskerException) {
-  BOOST_CHECK_THROW( {
-      MTasker<> mt;
-      mt.makeThread(willThrow, 0);
-      struct timeval now;
-      
-      for(;;) {
-	mt.schedule(&now);
-      }
-    }, std::exception);
+    for (;;) {
+      mt.schedule(&now);
+    }
+  },
+    std::exception);
 }
 BOOST_AUTO_TEST_SUITE_END()
